@@ -25,9 +25,19 @@
  *
  */
 
-#include <sys/wait.h>
 #include <errno.h>
 
+#include "portability.h"
+
+#ifdef LINUX
+#include <sys/wait.h>
+#endif
+
+#ifdef WINDOWS
+#include <stdio.h>
+#include <process.h>
+#endif
+
 #include "berusky.h"
 #include "berusky_gui.h"
 #include "main.h"
@@ -201,8 +211,6 @@ void game_gui::menu_password(MENU_STATE state, int data, int data1)
 {
   #define MAX_PASSWORD 10
   static char password[MAX_PASSWORD] = "";
-  static int curLevelSet = 0;
-  static int curLevel = 0;
 
   switch(state) {
     case MENU_RETURN:
@@ -250,71 +258,30 @@ void game_gui::menu_password(MENU_STATE state, int data, int data1)
         bool character = key_to_ascii(data,&ascii_char);
         int  len = strlen(password);
       
-        switch(data) {
-#ifdef HILDON
-          case K_F7: // next level
-            {
-              // printf("F7: curLevelSet=%d, curLevel=%d\n", curLevelSet, curLevel);
-              p_ber->levelset_load(curLevelSet);
-              if((curLevel + 1) < p_ber->levelset_get_levelnum()) {
-                curLevel++;
-              } else if((curLevelSet + 1) < LEVELSET_NUM) {
-                curLevelSet++;
-                p_ber->levelset_load(curLevelSet);
-                curLevel = 0;
+        if(character && ascii_char >= ' ') {
+          if(len < MAX_PASSWORD-2) {
+            password[len] = ascii_char;
+            password[len+1] = '\0';
+          }        
+        } else {
+          switch(data) {
+            case K_BKSP:
+            case K_DEL:
+              if(len > 0) {
+                password[--len] = '\0';
               }
-              char * pw = p_ber->levelset_get_passwd(curLevel);
-              if(pw!=NULL)
-                strncpy(password,pw,MAX_PASSWORD-1);
-              password[MAX_PASSWORD-1] = '\0';
-            }
-            break;
-          case K_F8: // previous level
-            {
-              // printf("F7: curLevelSet=%d, curLevel=%d\n", curLevelSet, curLevel);
-              p_ber->levelset_load(curLevelSet);
-              if(curLevel > 0) {
-                curLevel--;
-              } else if(curLevelSet > 0) {
-                curLevelSet--;
-                p_ber->levelset_load(curLevelSet);
-                curLevel = p_ber->levelset_get_levelnum() - 1;
-              }
-              char * pw = p_ber->levelset_get_passwd(curLevel);
-              if(pw!=NULL)
-                strncpy(password,pw,MAX_PASSWORD-1);
-              password[MAX_PASSWORD-1] = '\0';
-            }
-            break;
-#endif
-          case K_BKSP:
-          case K_DEL:
-            if(len > 0) {
-              password[--len] = '\0';
-            }
-            break;
-          case K_ESC:
-          case K_ENTER:
-            break;
-          default:
-            // moved this code in here for meamo since some hardeare keys are interpreted as 'A'
-            if(character) {
-              if(len < MAX_PASSWORD-2) {
-                password[len] = ascii_char;
-                password[len+1] = '\0';
-              }        
-            }
-            break;
+              break;
+            case K_ENTER:
+              break;
+            default:
+              break;
+          }
         }
       
         p_font->select(FONT_DEFAULT);
         p_font->alignment_set(CENTER);        
         p_grf->fill(0,INSERT_START+50,GAME_RESOLUTION_X,20,0);
-#ifdef HILDON
-        p_font->print(NULL, 0, INSERT_START+50, "(%s,%s) %s_", p_ber->levelset_get_difficulty(curLevelSet), p_ber->levelset_get_name(curLevel), password);
-#else
         p_font->print(NULL, 0, INSERT_START+50, "%s_", password);
-#endif
         p_grf->redraw_add(0,INSERT_START+50,GAME_RESOLUTION_X,20);
         p_grf->flip();
       }
@@ -689,7 +656,7 @@ void game_gui::menu_help_keys(MENU_STATE state, int data, int data1)
         p_font->print(NULL, 220, 100, _("Game Controls"));
       
         p_font->alignment_set(LEFT);
-        p_font->print(NULL, 20,  140, _("Up to five bugs are available,\
+        p_font->print(NULL, 20,  140, _("Up to five bugs are available,\n\
 which can be controlled by these keys:"));
       
         #define Y_DIFF  25
@@ -701,7 +668,7 @@ which can be controlled by these keys:"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("Tab"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("N"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("D"));
-        p_font->print(NULL, 20,  start_y += Y_DIFF, _("ALT+X"));
+        p_font->print(NULL, 20,  start_y += Y_DIFF, _("CTRL+X"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("F1"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("F2"));
         p_font->print(NULL, 20,  start_y += Y_DIFF, _("F3"));
@@ -917,6 +884,7 @@ void game_gui::menu_level_run(MENU_STATE state, int data, int data1)
       
         bool ret = p_ber->levelset_load(set);
         assert(ret);
+        p_ber->levelset_set_level(level);
       
         p_grf->fill(0,0,GAME_RESOLUTION_X,GAME_RESOLUTION_Y,0);
         p_font->alignment_set(CENTER);
@@ -1473,7 +1441,8 @@ void game_gui::level_load(LEVEL_EVENT_QUEUE *p_queue)
 }
 
 void game_gui::run_editor(void)
-{    
+{ 
+#ifdef LINUX
   int pid = fork();
   if(!pid) {
     bprintf("%s -e",p_dir->game_binary_get());
@@ -1488,6 +1457,13 @@ void game_gui::run_editor(void)
     waitpid(pid,&status,0);
     bprintf("Pid %d done",pid);
   }
+#elif WINDOWS
+  bprintf("%s -e",p_dir->game_binary_get());  
+  int ret = _spawnl( _P_WAIT, p_dir->game_binary_get(),p_dir->game_binary_get(),"-e",NULL);
+  if(ret == -1) {
+    bprintf("Error: %s",strerror(errno));
+  }
+#endif
 }
 
 bool game_gui::callback(LEVEL_EVENT_QUEUE *p_queue, int frame)
