@@ -28,7 +28,8 @@
 /* Global input interface
 */
 #include "berusky.h" 
-#include "berusky_gui.h" 
+#include "berusky_gui.h"
+#include "editor.h"
 
 /* KB definitions
 */
@@ -40,10 +41,6 @@ EVENT_KEY game_key_array[] =
   {LEVEL_EVENT(GL_PLAYER_MOVE, 1, 0),LEVEL_EVENT(EV_NONE), K_RIGHT,0,0,0,KEY_GROUP_BLOCK, KEY_GROUP_BLOCK_MOVE},
 
   {LEVEL_EVENT(GL_PLAYER_SWITCH, SELECT_PLAYER_NEXT, 0),LEVEL_EVENT(EV_NONE), K_TAB, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-#ifdef HILDON
-  // Select for Tab
-  {LEVEL_EVENT(GL_PLAYER_SWITCH, SELECT_PLAYER_NEXT, 0),LEVEL_EVENT(EV_NONE), K_ENTER, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-#endif
 
   {LEVEL_EVENT(GL_PLAYER_SWITCH,  0, 0),LEVEL_EVENT(EV_NONE), K_1, 0,0,0,KEY_CLEAR_AFTER_PRESS},
   {LEVEL_EVENT(GL_PLAYER_SWITCH,  1, 0),LEVEL_EVENT(EV_NONE), K_2, 0,0,0,KEY_CLEAR_AFTER_PRESS},
@@ -52,14 +49,9 @@ EVENT_KEY game_key_array[] =
   {LEVEL_EVENT(GL_PLAYER_SWITCH,  4, 0),LEVEL_EVENT(EV_NONE), K_5, 0,0,0,KEY_CLEAR_AFTER_PRESS},
 
   {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(GC_MENU_IN_GAME),   K_ESC, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-#ifdef HILDON
-  // Menu for F1
-  {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(GC_MENU_HELP,TRUE), K_F4, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(GC_MENU_LEVEL_HINT,TRUE), K_F4, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-#else
   {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(GC_MENU_HELP,TRUE), K_F1, 0,0,0,KEY_CLEAR_AFTER_PRESS},
   {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(GC_MENU_LEVEL_HINT,TRUE), K_F1, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-#endif
+
   {LEVEL_EVENT(GC_SUSPEND_LEVEL),LEVEL_EVENT(EV_NONE),K_S, 0,0,0,KEY_CLEAR_AFTER_PRESS},
   {LEVEL_EVENT(GC_RESTART_LEVEL),LEVEL_EVENT(EV_NONE),K_R, 0,1,0,KEY_CLEAR_AFTER_PRESS},
 
@@ -101,26 +93,33 @@ EVENT_KEY_SET menu_keys = { menu_key_array, sizeof(menu_key_array) / sizeof(menu
 */
 EVENT_KEY editor_key_array[] = 
 {
-  {LEVEL_EVENT(GC_MENU_QUIT),                             LEVEL_EVENT(EV_NONE),K_ESC,0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_GRID,LAYER_CHANGE),   LEVEL_EVENT(EV_NONE),K_1, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_FLOOR,LAYER_CHANGE),  LEVEL_EVENT(EV_NONE),K_2, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_ITEMS,LAYER_CHANGE),  LEVEL_EVENT(EV_NONE),K_3, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_PLAYER,LAYER_CHANGE), LEVEL_EVENT(EV_NONE),K_4, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_NEW),                             LEVEL_EVENT(EV_NONE),K_N, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SAVE),                            LEVEL_EVENT(EV_NONE),K_F2,0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_LOAD),                            LEVEL_EVENT(EV_NONE),K_F3,0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_UNDO),                                  LEVEL_EVENT(EV_NONE),K_U, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_REDO),                                  LEVEL_EVENT(EV_NONE),K_R, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_RUN),                             LEVEL_EVENT(EV_NONE),K_F9,0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SHADER),                          LEVEL_EVENT(EV_NONE),K_S, 0,1,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_ROTATE_SELECTION),                      LEVEL_EVENT(EV_NONE),K_R, 0,0,1,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_RECTANGLE_SELECTION),             LEVEL_EVENT(EV_NONE),K_D, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_RECTANGLE_SELECTION,TRUE),        LEVEL_EVENT(EV_NONE),K_F, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_FLOOR),        LEVEL_EVENT(EV_NONE),K_1, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_ITEMS),        LEVEL_EVENT(EV_NONE),K_2, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_PLAYER),       LEVEL_EVENT(EV_NONE),K_3, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,ALL_LEVEL_LAYERS),   LEVEL_EVENT(EV_NONE),K_4, 0,0,0,KEY_CLEAR_AFTER_PRESS},
-  {LEVEL_EVENT(ED_LEVEL_CHANGE_BACKGROUND),               LEVEL_EVENT(EV_NONE),K_B, 0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(GC_MENU_QUIT),                               LEVEL_EVENT(EV_NONE),K_ESC, 0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_GRID,LAYER_CHANGE),     LEVEL_EVENT(EV_NONE),K_1,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_FLOOR,LAYER_CHANGE),    LEVEL_EVENT(EV_NONE),K_2,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_ITEMS,LAYER_CHANGE),    LEVEL_EVENT(EV_NONE),K_3,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_LAYER,LAYER_PLAYER,LAYER_CHANGE),   LEVEL_EVENT(EV_NONE),K_4,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_HELP),                                    LEVEL_EVENT(EV_NONE),K_F1,  0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_NEW),                               LEVEL_EVENT(EV_NONE),K_N,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SAVE),                              LEVEL_EVENT(EV_NONE),K_F2,  0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SAVE_AS),                           LEVEL_EVENT(EV_NONE),K_F2,  0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_LOAD),                              LEVEL_EVENT(EV_NONE),K_F3,  0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_UNDO),                                    LEVEL_EVENT(EV_NONE),K_U,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_REDO),                                    LEVEL_EVENT(EV_NONE),K_R,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_RUN),                               LEVEL_EVENT(EV_NONE),K_F9,  0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SHADER),                            LEVEL_EVENT(EV_NONE),K_S,   0,1,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_ROTATE_SELECTION),                        LEVEL_EVENT(EV_NONE),K_R,   0,0,1,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_RECTANGLE_SELECTION,FALSE),         LEVEL_EVENT(EV_NONE),K_D,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_RECTANGLE_SELECTION,TRUE),          LEVEL_EVENT(EV_NONE),K_F,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_FLOOR),          LEVEL_EVENT(EV_NONE),K_1,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_ITEMS),          LEVEL_EVENT(EV_NONE),K_2,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,LAYER_PLAYER),         LEVEL_EVENT(EV_NONE),K_3,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_SELECT_LAYER,ALL_LEVEL_LAYERS),     LEVEL_EVENT(EV_NONE),K_4,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_CHANGE_BACKGROUND),                 LEVEL_EVENT(EV_NONE),K_B,   0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_MOUSE_PANEL_SCROLL,-ITEMS_IN_PANEL),LEVEL_EVENT(EV_NONE),K_PGUP,0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_MOUSE_PANEL_SCROLL,ITEMS_IN_PANEL), LEVEL_EVENT(EV_NONE),K_PGDN,0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_MOUSE_PANEL_SCROLL,ITEMS_START),    LEVEL_EVENT(EV_NONE),K_HOME,0,0,0,KEY_CLEAR_AFTER_PRESS},
+  {LEVEL_EVENT(ED_LEVEL_MOUSE_PANEL_SCROLL,ITEMS_END),      LEVEL_EVENT(EV_NONE),K_END, 0,0,0,KEY_CLEAR_AFTER_PRESS},
+
   //{LEVEL_EVENT(EV_TEST),                                  LEVEL_EVENT(EV_NONE),K_A, 0,0,0,KEY_CLEAR_AFTER_PRESS},
 };
 
@@ -270,39 +269,26 @@ void input::events_loop(LEVEL_EVENT_QUEUE *p_queue)
         key_input(event.key.keysym.sym, event.key.keysym.mod, FALSE);
         break;
       case SDL_MOUSEMOTION:
-        printf("SDL_MOUSEMOTION\n");
         {
           bool pressed = FALSE;
           int  i;
           for(i = 0; i < MOUSE_BUTTONS; i++) {        
             bool state = event.motion.state&SDL_BUTTON(i);
             if(state) {
-              printf("  moved\n");
-#ifndef HILDON
-        // under Maemo this can cause two mouse clicks instead of one
-        // which results in unwanted behaviour in menus
               mouse_input(event.motion.x, event.motion.y, 1, i);
-#endif
               pressed = TRUE;
             }
           }
           if(!pressed) {
-            printf("  pressed\n");
             mouse_input(event.motion.x, event.motion.y, 0, 0);
           }
         }
         break;
       case SDL_MOUSEBUTTONDOWN:
-        printf("SDL_MOUSEBUTTONDOWN\n");
         mouse_input(event.button.x, event.button.y, 1, event.button.button);
         break;
       case SDL_MOUSEBUTTONUP:
-        printf("SDL_MOUSEBUTTONUP\n");
-#ifndef HILDON
-        // under Maemo this can cause two mouse clicks instead of one
-        // which results in unwanted behaviour in menus
         mouse_input(event.button.x, event.button.y, 0, event.button.button);
-#endif
         break;
       case SDL_ACTIVEEVENT:
         if(event.active.state&SDL_APPACTIVE) {
@@ -337,17 +323,6 @@ void input::events_loop(LEVEL_EVENT_QUEUE *p_queue)
 // Mouse interface
 // -------------------------------------------------------------------------
 
-inline bool in_rect(RECT &dst, tpos x, tpos y)
-{
-  return(x >= dst.x && y >= dst.y && x < dst.x+dst.w && y < dst.y+dst.h);
-}
-
-inline bool in_rect(RECT &dst, RECT &src)
-{
-  return(src.x >= dst.x && src.y >= dst.y && 
-         src.x < dst.x+dst.w && src.y < dst.y+dst.h);
-}
-
 void input::mouse_input(tpos mx, tpos my, int state, int button)
 {
   // bprintf("Mouse state [%d,%d], state %d, button %d", mx, my, state, button);
@@ -413,11 +388,10 @@ void input::mouse_input(tpos mx, tpos my, int state, int button)
       
       if(active) {
         if(p_ev->flag&MEVENT_MOUSE_EXTERNAL) {          
-          if(p_ev->flag&MEVENT_MOUSE_BUTTONS) {
+          if(p_ev->flag&MEVENT_MOUSE_BUTTONS)
             input_queue.add(LEVEL_EVENT(GI_MOUSE_EVENT, p_ev->event[0].param_int_get(PARAM_0), mx, my, state, button));
-          } else {
+          else
             input_queue.add(LEVEL_EVENT(GI_MOUSE_EVENT, p_ev->event[0].param_int_get(PARAM_0), mx, my, 0, 0));
-          }
         }
         else {
           input_queue.add(p_ev->event,p_ev->event_num);
