@@ -201,19 +201,28 @@ typedef unsigned short int word;
 
 #define H_LEVEL  "Level 3 (C) Anakreon 1999"
 
+#ifdef LINUX
 typedef struct __attribute__ ((__packed__)) {
+#elif WINDOWS
+#pragma pack(push,1)
+typedef struct {
+#endif
 
-   char signum[30];               // -> retezec "Berusky (C) Anakreon 1998"
-   char back;                     // -> cislo pozadi
-   char music;                    // -> cislo hudby k tomuto levelu
-   char rot[5];                   // -> natoceni hracu
-   char rezerved[100];            // -> pro pozdejsi pouziti
-   word floor[LEVEL_CELLS_Y][LEVEL_CELLS_X][10];// -> podlaha
-   word level[LEVEL_CELLS_Y][LEVEL_CELLS_X][10];  //-> herni plocha (dnesni level)
-   word players[LEVEL_CELLS_Y][LEVEL_CELLS_X];    //-> mapa hracu (dneska sou hraci n levelu)
+   char signum[30];               // -> "Berusky (C) Anakreon 1998"
+   char back;                     // -> background number
+   char music;                    // -> track number
+   char rot[5];                   // -> players rotation
+   char rezerved[100];            // -> reserved
+   word floor[LEVEL_CELLS_Y][LEVEL_CELLS_X][10];  //-> floor cells
+   word level[LEVEL_CELLS_Y][LEVEL_CELLS_X][10];  //-> level items
+   word players[LEVEL_CELLS_Y][LEVEL_CELLS_X];    //-> players
 
 } LEVEL_DISK;
 
+#ifdef WINDOWS
+#pragma pack(pop)
+#endif
+
 typedef struct level_cell {
 
   item_handle   item;       // -> 
@@ -367,10 +376,10 @@ public:
   void level_clear(void);
 
   bool level_new(void);
-  bool level_load(char *p_file);
-  bool level_load(char *p_file, char **p_dirs, int dirs);
-  bool level_save(char *p_file);
-  bool level_exists(char *p_file);
+  bool level_load(const char *p_file);
+  bool level_load(const char *p_file, const char **p_dirs, int dirs);
+  bool level_save(const char *p_file);
+  bool level_exists(const char *p_file);
 
   void level_import(LEVEL_DISK *p_lev);
   LEVEL_DISK * level_export(LEVEL_DISK *p_lev);
